# This file has been generated


build_system = 'AppVeyor'
build_id = '29930071'
build_number = '435'
build_version = '1.0.435'
job_id = 'ujeqneva89n208bx'
job_url = 'https://ci.appveyor.com/project/KyleAltendorf/stlib/build/job/ujeqneva89n208bx'
