# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'device.ui'
#
# Created by: PyQt5 UI code generator 5.13.0
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(824, 809)
        self.verticalLayout = QtWidgets.QVBoxLayout(Form)
        self.verticalLayout.setObjectName("verticalLayout")
        self.overlay_stack = QtWidgets.QStackedWidget(Form)
        self.overlay_stack.setObjectName("overlay_stack")
        self.overlays = QtWidgets.QWidget()
        self.overlays.setObjectName("overlays")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.overlays)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.offline_overlay = OverlayLabel(self.overlays)
        self.offline_overlay.setObjectName("offline_overlay")
        self.gridLayout_3.addWidget(self.offline_overlay, 0, 0, 1, 1)
        self.connection_monitor_overlay = OverlayLabel(self.overlays)
        self.connection_monitor_overlay.setObjectName("connection_monitor_overlay")
        self.gridLayout_3.addWidget(self.connection_monitor_overlay, 1, 0, 1, 1)
        self.overlay_stack.addWidget(self.overlays)
        self.main_stack = QtWidgets.QWidget()
        self.main_stack.setObjectName("main_stack")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.main_stack)
        self.gridLayout_2.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.tabs = QtWidgets.QTabWidget(self.main_stack)
        self.tabs.setObjectName("tabs")
        self.txrx = QtWidgets.QWidget()
        self.txrx.setObjectName("txrx")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.txrx)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.splitter = QtWidgets.QSplitter(self.txrx)
        self.splitter.setOrientation(QtCore.Qt.Vertical)
        self.splitter.setObjectName("splitter")
        self.layoutWidget = QtWidgets.QWidget(self.splitter)
        self.layoutWidget.setObjectName("layoutWidget")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.layoutWidget)
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.label = QtWidgets.QLabel(self.layoutWidget)
        self.label.setAlignment(QtCore.Qt.AlignCenter)
        self.label.setObjectName("label")
        self.verticalLayout_2.addWidget(self.label)
        self.tx = TxRxView(self.layoutWidget)
        self.tx.setObjectName("tx")
        self.verticalLayout_2.addWidget(self.tx)
        self.layoutWidget_2 = QtWidgets.QWidget(self.splitter)
        self.layoutWidget_2.setObjectName("layoutWidget_2")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.layoutWidget_2)
        self.verticalLayout_5.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.label_2 = QtWidgets.QLabel(self.layoutWidget_2)
        self.label_2.setAlignment(QtCore.Qt.AlignCenter)
        self.label_2.setObjectName("label_2")
        self.verticalLayout_5.addWidget(self.label_2)
        self.rx = TxRxView(self.layoutWidget_2)
        self.rx.setObjectName("rx")
        self.verticalLayout_5.addWidget(self.rx)
        self.verticalLayout_3.addWidget(self.splitter)
        self.tabs.addTab(self.txrx, "")
        self.variables = QtWidgets.QWidget()
        self.variables.setObjectName("variables")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.variables)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.variable_selection = VariableSelection(self.variables)
        self.variable_selection.setObjectName("variable_selection")
        self.verticalLayout_6.addWidget(self.variable_selection)
        self.tabs.addTab(self.variables, "")
        self.nv = QtWidgets.QWidget()
        self.nv.setObjectName("nv")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.nv)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.nv1 = NvView(self.nv)
        self.nv1.setObjectName("nv1")
        self.verticalLayout_4.addWidget(self.nv1)
        self.tabs.addTab(self.nv, "")
        self.scripting = QtWidgets.QWidget()
        self.scripting.setObjectName("scripting")
        self.gridLayout = QtWidgets.QGridLayout(self.scripting)
        self.gridLayout.setObjectName("gridLayout")
        self.scripting_view = ScriptingView(self.scripting)
        self.scripting_view.setObjectName("scripting_view")
        self.gridLayout.addWidget(self.scripting_view, 0, 0, 1, 1)
        self.tabs.addTab(self.scripting, "")
        self.faultlog = QtWidgets.QWidget()
        self.faultlog.setObjectName("faultlog")
        self.gridLayout1 = QtWidgets.QGridLayout(self.faultlog)
        self.gridLayout1.setObjectName("gridLayout1")
        self.fault_log_view = FaultLogView(self.faultlog)
        self.fault_log_view.setObjectName("fault_log_view")
        self.gridLayout1.addWidget(self.fault_log_view, 0, 0, 1, 1)
        self.tabs.addTab(self.faultlog, "")
        self.files = QtWidgets.QWidget()
        self.files.setObjectName("files")
        self.gridLayout2 = QtWidgets.QGridLayout(self.files)
        self.gridLayout2.setObjectName("gridLayout2")
        self.files_view = FilesViewQtBuilder(self.files)
        self.files_view.setObjectName("files_view")
        self.gridLayout2.addWidget(self.files_view, 0, 0, 1, 1)
        self.tabs.addTab(self.files, "")
        self.gridLayout_2.addWidget(self.tabs, 0, 0, 1, 1)
        self.overlay_stack.addWidget(self.main_stack)
        self.verticalLayout.addWidget(self.overlay_stack)

        self.retranslateUi(Form)
        self.tabs.setCurrentIndex(1)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.offline_overlay.setProperty("text", _translate("Form", "offline"))
        self.label.setText(_translate("Form", "Transmit"))
        self.label_2.setText(_translate("Form", "Receive"))
        self.tabs.setTabText(self.tabs.indexOf(self.txrx), _translate("Form", "Tx/Rx"))
        self.tabs.setTabText(self.tabs.indexOf(self.variables), _translate("Form", "Data Logger"))
        self.tabs.setTabText(self.tabs.indexOf(self.nv), _translate("Form", "Parameters"))
        self.tabs.setTabText(self.tabs.indexOf(self.scripting), _translate("Form", "Scripting"))
        self.tabs.setTabText(self.tabs.indexOf(self.faultlog), _translate("Form", "Fault Log"))
        self.tabs.setTabText(self.tabs.indexOf(self.files), _translate("Form", "Files"))
from epyqlib.faultlogview import FaultLogView
from epyqlib.filesview import FilesViewQtBuilder
from epyqlib.nvview import NvView
from epyqlib.overlaylabel import OverlayLabel
from epyqlib.scriptingview import ScriptingView
from epyqlib.txrxview import TxRxView
from epyqlib.variableselection import VariableSelection
